@@ -5,17 +5,18 @@ __author__ = "giacomov"
 import collections
 import os
 import warnings
+from dataclasses import dataclass
 from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
 
 import numpy as np
 import pandas as pd
 import scipy.integrate
-
 from astromodels.core.memoization import use_astromodels_memoization
 from astromodels.core.my_yaml import my_yaml
 from astromodels.core.parameter import IndependentVariable, Parameter
+from astromodels.core.property import FunctionProperty
 from astromodels.core.tree import DuplicatedNode, Node
-from astromodels.functions.function import get_function
+from astromodels.functions.function import Function, get_function
 from astromodels.sources import (ExtendedSource, ParticleSource, PointSource,
                                  Source, SourceType)
 from astromodels.utils.disk_usage import disk_usage
@@ -52,6 +53,32 @@ class ModelInternalError(ValueError):
     pass
 
 
+@dataclass(frozen=True)
+class _LinkedFunctionContainer:
+    function_name: str
+    linked_path: str
+    internal_name: str
+
+    def output(self, rich=True):
+
+        this_dict = collections.OrderedDict()
+
+        tmp = collections.OrderedDict()
+        tmp["linked function"] = self.linked_path
+        tmp["internal name"] = self.internal_name
+        this_dict[self.function_name] = tmp
+
+        output = pd.DataFrame.from_dict(this_dict)
+
+        if rich:
+
+            return output._repr_html_()
+
+        else:
+
+            return output.__repr__()
+
+
 class Model(Node):
     def __init__(self, *sources):
 
@@ -65,13 +92,11 @@ class Model(Node):
 
         # Dictionary to keep extended sources
 
-        self._extended_sources: Dict[
-            str, ExtendedSource] = collections.OrderedDict()
+        self._extended_sources: Dict[str, ExtendedSource] = collections.OrderedDict()
 
         # Dictionary to keep particle sources
 
-        self._particle_sources: Dict[
-            str, ParticleSource] = collections.OrderedDict()
+        self._particle_sources: Dict[str, ParticleSource] = collections.OrderedDict()
 
         # Loop over the provided sources and process them
 
@@ -90,8 +115,8 @@ class Model(Node):
         self._independent_variables = {}
 
     def _add_source(
-            self, source: Union[PointSource, ExtendedSource,
-                                ParticleSource]) -> None:
+        self, source: Union[PointSource, ExtendedSource, ParticleSource]
+    ) -> None:
         """
         Remember to call _update_parameters after this!
         :param source:
@@ -108,7 +133,8 @@ class Model(Node):
 
                 log.error(
                     "More than one source with the name '%s'. You cannot use the same name for multiple "
-                    "sources" % source.name)
+                    "sources" % source.name
+                )
 
                 raise DuplicatedNode()
 
@@ -168,29 +194,16 @@ class Model(Node):
 
     def _find_parameters(self, node) -> Dict[str, Parameter]:
 
-        instances = collections.OrderedDict()
+        return self._recursively_gather_node_type(self, Parameter)
 
-        for child in node._get_children():
+    def _find_properties(self, node) -> Dict[str, FunctionProperty]:
 
-            if isinstance(child, Parameter):
-
-                path = child._get_path()
-
-                instances[path] = child
-
-                for sub_child in child._get_children():
-
-                    instances.update(self._find_parameters(sub_child))
-
-            else:
-
-                instances.update(self._find_parameters(child))
-
-        return instances
+        return self._recursively_gather_node_type(self, FunctionProperty)
 
     def _update_parameters(self) -> None:
 
         self._parameters: Dict[str, Parameter] = self._find_parameters(self)
+        self._properties: Dict[str, FunctionProperty] = self._find_properties(self)
 
     @property
     def parameters(self) -> Dict[str, Parameter]:
@@ -217,8 +230,7 @@ class Model(Node):
 
         # Filter selecting only free parameters
 
-        free_parameters_dictionary: Dict[
-            str, Parameter] = collections.OrderedDict()
+        free_parameters_dictionary: Dict[str, Parameter] = collections.OrderedDict()
 
         for parameter_name, parameter in list(self._parameters.items()):
 
@@ -244,17 +256,88 @@ class Model(Node):
 
         # Filter selecting only free parameters
 
-        linked_parameter_dictionary: Dict[
-            str, Parameter] = collections.OrderedDict()
+        linked_parameter_dictionary: Dict[str, Parameter] = collections.OrderedDict()
 
         for parameter_name, parameter in list(self._parameters.items()):
 
-            if parameter.has_auxiliary_variable():
+            if parameter.has_auxiliary_variable:
 
                 linked_parameter_dictionary[parameter_name] = parameter
 
         return linked_parameter_dictionary
 
+    @property
+    def properties(self) -> Dict[str, Parameter]:
+        """
+        Return a dictionary with all parameters
+
+        :return: dictionary of parameters
+        """
+        self._update_parameters()
+
+        return self._properties
+
+    @property
+    def linked_functions(self) -> List[_LinkedFunctionContainer]:
+        """
+        return a list of containers for the linked functions
+
+        """
+
+        linked_functions = []
+
+        for function in self._get_all_functions():
+
+            if "composite" in function._children:
+
+                data = function.to_dict()["composite"]
+
+                # this is a composite
+
+                if "external_functions" in data:
+
+                    for k, v in data["external_functions"].items():
+
+                        if v:
+
+                            for name, path in v.items():
+
+                                tmp = _LinkedFunctionContainer(
+                                    function_name=function.path,
+                                    linked_path=path,
+                                    internal_name=name,
+                                )
+
+                                linked_functions.append(tmp)
+            else:
+
+                data = function.to_dict()[function.shape.name]
+
+                if "external_functions" in data:
+
+                    for name, path in data["external_functions"].items():
+
+                        tmp = _LinkedFunctionContainer(
+                            function_name=function.path,
+                            linked_path=path,
+                            internal_name=name,
+                        )
+
+                        linked_functions.append(tmp)
+
+        return linked_functions
+
+    def _get_all_functions(self) -> List[Function]:
+        all_functions = []
+
+        for source_name, source in self.sources.items():
+
+            # look at the spectrum node
+
+            all_functions.extend(source.spectrum._get_children())
+
+        return all_functions
+
     def set_free_parameters(self, values: Iterable[float]) -> None:
         """
         Set the free parameters in the model to the provided values.
@@ -273,8 +356,7 @@ class Model(Node):
 
             raise AssertionError()
 
-        for parameter, this_value in zip(list(self.free_parameters.values()),
-                                         values):
+        for parameter, this_value in zip(list(self.free_parameters.values()), values):
 
             parameter.value = this_value
 
@@ -360,9 +442,7 @@ class Model(Node):
         return self._particle_sources
 
     @property
-    def sources(
-            self
-    ) -> Dict[str, Union[PointSource, ExtendedSource, ParticleSource]]:
+    def sources(self) -> Dict[str, Union[PointSource, ExtendedSource, ParticleSource]]:
         """
         Returns a dictionary containing all defined sources (of any kind)
 
@@ -370,19 +450,19 @@ class Model(Node):
 
         """
 
-        sources: Dict[str, Union[PointSource, ExtendedSource,
-                                 ParticleSource]] = collections.OrderedDict()
+        sources: Dict[
+            str, Union[PointSource, ExtendedSource, ParticleSource]
+        ] = collections.OrderedDict()
 
-        for d in (self.point_sources, self.extended_sources,
-                  self.particle_sources):
+        for d in (self.point_sources, self.extended_sources, self.particle_sources):
 
             sources.update(d)
 
         return sources
 
     def add_source(
-        self, new_source: Union[PointSource, ExtendedSource,
-                                ParticleSource]) -> None:
+        self, new_source: Union[PointSource, ExtendedSource, ParticleSource]
+    ) -> None:
         """
         Add the provided source to the model
 
@@ -408,9 +488,7 @@ class Model(Node):
 
         self._update_parameters()
 
-    def unlink_all_from_source(self,
-                               source_name: str,
-                               warn: bool = False) -> None:
+    def unlink_all_from_source(self, source_name: str, warn: bool = False) -> None:
         """
         Unlink all parameters of the current model that are linked to a parameter of a given source.
         To be called before removing a source from the model.
@@ -424,7 +502,7 @@ class Model(Node):
 
         for par in self.linked_parameters.values():
 
-            target = par._aux_variable['variable']
+            target = par._aux_variable["variable"]
 
             if target.path in tempmodel:
 
@@ -434,10 +512,12 @@ class Model(Node):
         if warn and unlinked_parameters:
 
             log.warning(
-                "The following %d parameters that were linked to source %s have been automatically un-linked: %s"
-                % (len(unlinked_parameters), source_name,
-                   [p.path for p in unlinked_parameters.values()]),
-                RuntimeWarning)
+                f"The following {len(unlinked_parameters)} parameters that were linked to source {source_name}"
+            )
+
+            log.warning(
+                f"have been automatically un-linked: {[p.path for p in unlinked_parameters.values()]}"
+            )
 
     def add_independent_variable(self, variable: IndependentVariable) -> None:
         """
@@ -498,7 +578,8 @@ class Model(Node):
 
                 log.warning(
                     "External parameter %s already exist in the model. Overwriting it..."
-                    % parameter.name)
+                    % parameter.name
+                )
 
                 self._remove_child(parameter.name)
 
@@ -537,16 +618,16 @@ class Model(Node):
         for param_1 in parameter_1_list:
             if not param_1.path in self:
 
-                log.error( "Parameter %s is not contained in this model" % param_1.path )
+                log.error("Parameter %s is not contained in this model" % param_1.path)
 
                 raise AssertionError()
-                
+
         if not parameter_2.path in self:
 
-            log.error( "Parameter %s is not contained in this model" % parameter_2.path )
+            log.error("Parameter %s is not contained in this model" % parameter_2.path)
 
             raise AssertionError()
-            
+
         if link_function is None:
             # Use the Line function by default, with both parameters fixed so that the two
             # parameters to be linked will vary together
@@ -579,7 +660,7 @@ class Model(Node):
             parameter_list = list(parameter)
 
         for param in parameter_list:
-            if param.has_auxiliary_variable():
+            if param.has_auxiliary_variable:
                 param.remove_auxiliary_variable()
 
             else:
@@ -588,12 +669,9 @@ class Model(Node):
 
                     warnings.simplefilter("always", RuntimeWarning)
 
-                    log.warning(
-                        "Parameter %s has no link to be removed." % param.path
-                       
-                    )
+                    log.warning("Parameter %s has no link to be removed." % param.path)
 
-    def display(self, complete: bool=False) -> None:
+    def display(self, complete: bool = False) -> None:
         """
         Display information about the point source.
 
@@ -639,6 +717,8 @@ class Model(Node):
         parameters = self.parameters
         free_parameters = self.free_parameters
         linked_parameters = self.linked_parameters
+        properties = self.properties
+        linked_functions = self.linked_functions
 
         # Summary of free parameters
         if len(free_parameters) > 0:
@@ -751,6 +831,42 @@ class Model(Node):
 
         empty_frame = "(none)%s" % new_line
 
+        # Summary of free parameters
+        if len(properties) > 0:
+
+            property_dict = collections.OrderedDict()
+
+            for property_name, prop in properties.items():
+
+                # Generate table with only a minimal set of info
+                if rich_output:
+
+                    this_name = long_path_formatter(property_name, 70)
+
+                else:
+
+                    # In a terminal we need to use less characters
+
+                    this_name = long_path_formatter(property_name, 40)
+
+                d = prop.to_dict()
+                property_dict[this_name] = collections.OrderedDict()
+
+                for key in ["value", "allowed values"]:
+
+                    property_dict[this_name][key] = d[key]
+
+            properties_summary = pd.DataFrame.from_dict(property_dict).T
+
+            # Re-order it
+            properties_summary = properties_summary[["value", "allowed values"]]
+
+        else:
+
+            properties_summary = pd.DataFrame()
+
+        empty_frame = "(none)%s" % new_line
+
         # Independent variables
 
         independent_v_frames = []
@@ -807,6 +923,29 @@ class Model(Node):
                     linked_summary_representation += linked_frame._repr_html_()
                     linked_summary_representation += new_line
 
+            if properties_summary.empty:
+
+                properties_representation = empty_frame
+
+            else:
+
+                properties_representation = properties_summary._repr_html_()
+
+            if len(linked_functions) == 0:
+
+                linked_function_summary_representation = empty_frame
+
+            else:
+
+                linked_function_summary_representation = ""
+
+                for linked_function in linked_functions:
+
+                    linked_function_summary_representation += linked_function.output(
+                        rich=True
+                    )
+                    linked_function_summary_representation += new_line
+
             if len(independent_v_frames) == 0:
 
                 independent_v_representation = empty_frame
@@ -840,6 +979,14 @@ class Model(Node):
 
                 free_parameters_representation = free_parameters_summary.__repr__()
 
+            if properties_summary.empty:
+
+                properties_representation = empty_frame
+
+            else:
+
+                properties_representation = properties_summary.__repr__()
+
             if len(linked_frames) == 0:
 
                 linked_summary_representation = empty_frame
@@ -853,6 +1000,24 @@ class Model(Node):
                     linked_summary_representation += linked_frame.__repr__()
                     linked_summary_representation += "%s%s" % (new_line, new_line)
 
+            if len(linked_functions) == 0:
+
+                linked_function_summary_representation = empty_frame
+
+            else:
+
+                linked_function_summary_representation = ""
+
+                for linked_function in linked_functions:
+
+                    linked_function_summary_representation += linked_function.output(
+                        rich=False
+                    )
+                    linked_function_summary_representation += "%s%s" % (
+                        new_line,
+                        new_line,
+                    )
+
             if len(independent_v_frames) == 0:
 
                 independent_v_representation = empty_frame
@@ -938,6 +1103,26 @@ class Model(Node):
 
         representation += new_line
 
+        # Properties
+
+        representation += "%sProperties (%i):%s" % (
+            new_line,
+            len(properties),
+            new_line,
+        )
+
+        if not rich_output:
+
+            representation += "--------------------%s%s" % (new_line, new_line)
+
+        else:
+
+            representation += new_line
+
+        representation += properties_representation
+
+        representation += new_line
+
         # Linked parameters
 
         representation += "%sLinked parameters (%i):%s" % (
@@ -970,6 +1155,24 @@ class Model(Node):
 
         representation += independent_v_representation
 
+        # Linked functions
+
+        representation += "%sLinked functions (%i):%s" % (
+            new_line,
+            len(self.linked_functions),
+            new_line,
+        )
+
+        if not rich_output:
+
+            representation += "----------------------%s%s" % (new_line, new_line)
+
+        else:
+
+            representation += new_line
+
+        representation += linked_function_summary_representation
+
         return representation
 
     def to_dict_with_types(self):
@@ -1009,6 +1212,10 @@ class Model(Node):
 
                     data["%s (%s)" % (key, "Parameter")] = data.pop(key)
 
+                # elif isinstance(element, FunctionProperty):
+
+                #     data["%s (%s)" % (key, "FunctionProperty")] = data.pop(key)
+
                 else:  # pragma: no cover
 
                     raise ModelInternalError("Found an unknown class at the top level")
@@ -1053,7 +1260,7 @@ class Model(Node):
                     "report on the free space which follows: " % output_file,
                 )
 
-    def get_number_of_point_sources(self) ->int:
+    def get_number_of_point_sources(self) -> int:
         """
         Return the number of point sources
 
@@ -1073,7 +1280,9 @@ class Model(Node):
 
         return pts.position.get_ra(), pts.position.get_dec()
 
-    def get_point_source_fluxes(self, id: int, energies: np.ndarray, tag=None) -> np.ndarray:
+    def get_point_source_fluxes(
+        self, id: int, energies: np.ndarray, tag=None
+    ) -> np.ndarray:
         """
         Get the fluxes from the id-th point source
 
@@ -1086,7 +1295,7 @@ class Model(Node):
         set to a and the model evaluated in a.
         :return: fluxes
         """
-        
+
         return list(self._point_sources.values())[id](energies, tag=tag)
 
     def get_point_source_name(self, id: int) -> str:
@@ -1101,7 +1310,9 @@ class Model(Node):
         """
         return len(self._extended_sources)
 
-    def get_extended_source_fluxes(self, id: int, j2000_ra: float, j2000_dec: float, energies: np.ndarray) -> np.ndarray:
+    def get_extended_source_fluxes(
+        self, id: int, j2000_ra: float, j2000_dec: float, energies: np.ndarray
+    ) -> np.ndarray:
         """
         Get the flux of the id-th extended sources at the given position at the given energies
 
